ProgramError

Enum ProgramError 

pub enum ProgramError {
Show 26 variants Custom(u32), InvalidArgument, InvalidInstructionData, InvalidAccountData, AccountDataTooSmall, InsufficientFunds, IncorrectProgramId, MissingRequiredSignature, AccountAlreadyInitialized, UninitializedAccount, NotEnoughAccountKeys, AccountBorrowFailed, MaxSeedLengthExceeded, InvalidSeeds, BorshIoError, AccountNotRentExempt, UnsupportedSysvar, IllegalOwner, MaxAccountsDataAllocationsExceeded, InvalidRealloc, MaxInstructionTraceLengthExceeded, BuiltinProgramsMustConsumeComputeUnits, InvalidAccountOwner, ArithmeticOverflow, Immutable, IncorrectAuthority,
}
Reasons the program may fail

Variants



Custom(u32)

Allows on-chain programs to implement program-specific error types and see them returned by the Solana runtime. A program-specific error may be any type that is represented as or serialized to a u32 integer.



InvalidArgument



InvalidInstructionData



InvalidAccountData



AccountDataTooSmall



InsufficientFunds



IncorrectProgramId



MissingRequiredSignature



AccountAlreadyInitialized



UninitializedAccount



NotEnoughAccountKeys



AccountBorrowFailed



MaxSeedLengthExceeded



InvalidSeeds



BorshIoError



AccountNotRentExempt



UnsupportedSysvar



IllegalOwner



MaxAccountsDataAllocationsExceeded



InvalidRealloc



MaxInstructionTraceLengthExceeded



BuiltinProgramsMustConsumeComputeUnits



InvalidAccountOwner



ArithmeticOverflow



Immutable



IncorrectAuthority

Implementations

Source

impl ProgramError

pub fn to_str<E>(&self) -> &'static str
where E: 'static + ToStr + TryFrom<u32>,

Get an appropriate error string given a program error and an expected error type, if the error implements TryFrom<u32> and ToStr.

Example
#[derive(num_enum::TryFromPrimitive)]
#[repr(u32)]
enum MyError {
    A,
    B,
}

impl solana_program_error::ToStr for MyError {
    fn to_str(&self) -> &'static str {
        match self {
            MyError::A => "Message for A",
            MyError::B => "Some other message for B",
        }
    }
}

let program_error = solana_program_error::ProgramError::Custom(1);
assert_eq!("Some other message for B", program_error.to_str::<MyError>());