pub struct Address(/* private fields */);Expand description
The address of a Solana account.
Some account addresses are ed25519 public keys, with corresponding secret
keys that are managed off-chain. Often, though, account addresses do not
have corresponding secret keys — as with program derived
addresses — or the secret key is not relevant to the operation
of a program, and may have even been disposed of. As running Solana programs
can not safely create or manage secret keys, the full Keypair is not
defined in solana-program but in solana-sdk.
Implementations
Sourceimpl Address
impl Address
pub const fn new_from_array(address_array: [u8; 32]) -> Address
pub const fn to_bytes(&self) -> [u8; 32]
Sourcepub fn is_on_curve(&self) -> bool
pub fn is_on_curve(&self) -> bool
Checks whether the given address lies on the Ed25519 curve.
On-curve addresses correspond to valid Ed25519 public keys (and therefore can have associated private keys). Off-curve addresses are typically used for program-derived addresses (PDAs).